(function (Drupal) {
  Drupal.behaviors.imageFadingCarousel = {
    attach: function (context, settings) {
      const carousels = once('image-fading-carousel', '.image-fading-carousel', context);
      carousels.forEach(carousel => {
        let carouselName = carousel.getAttribute('data-carousel-name');

        // Skip carousel displaying when no data available.
        if (
          !settings.image_fading_carousel[carouselName]
          || !(settings.image_fading_carousel[carouselName].hasOwnProperty('change_interval'))
          || !(settings.image_fading_carousel[carouselName].hasOwnProperty('max_images_to_display'))
          || !(settings.image_fading_carousel[carouselName].hasOwnProperty('image_list_urls'))
          || !(settings.image_fading_carousel[carouselName].image_list_urls instanceof Array)
          || settings.image_fading_carousel[carouselName].image_list_urls.length === 0
        ) {
          return;
        }

        let changeIntervalSeconds = 5;
        const changeIntervalSetting = Number(settings.image_fading_carousel[carouselName].change_interval);
        if (Number.isInteger(changeIntervalSetting)) {
          changeIntervalSeconds = changeIntervalSetting;
        }
        const changeInterval = changeIntervalSeconds * 1000;

        let maxImagesToDisplay = 1;
        const maxImagesToDisplaySetting = Number(settings.image_fading_carousel[carouselName].max_images_to_display);
        if (Number.isInteger(maxImagesToDisplaySetting)) {
          maxImagesToDisplay = maxImagesToDisplaySetting;
        }

        const imageListUrls = settings.image_fading_carousel[carouselName].image_list_urls;

        // Insert needed structure to hold images.
        const displayableImagesCount = Math.min(maxImagesToDisplay, imageListUrls.length);
        const availableImagesCount = imageListUrls.length;
        const ul = document.createElement('ul');
        for (let i = 0; i < displayableImagesCount; i++) {
          const li = document.createElement('li');
          const img = document.createElement('img');
          img.src = ''; // Set empty src attribute
          li.appendChild(img);
          ul.appendChild(li);
        }
        carousel.appendChild(ul);

        const imgElements = [...document.querySelectorAll(`.image-fading-carousel[data-carousel-name="${carouselName}"] ul li img`)];

        // Array of unique images numbers.
        let visibleImages = [];

        // Set initial images set (random image set each time page load).
        for (const [i, imgElement] of imgElements.entries()) {
          // Push unique number to `visibleImages`.
          setUniqueNumbersArray();

          // Assign url to image `src` attribute.
          imgElement.src = imageListUrls[visibleImages[i]];
        }

        /**
         * Generate a unique random number and add it to the `visibleImages` array.
         */
        function setUniqueNumbersArray() {
          // Generate random number with limit of `imageListUrls.length`
          let randomNumber = Math.floor(Math.random() * imageListUrls.length);

          // Check if current generated number is in `visibleImages` array.
          // If true run function `setUniqueNumbersArray` again.
          // If false add number into `visibleImages` array.
          visibleImages.includes(randomNumber)
            ? setUniqueNumbersArray()
            : visibleImages.push(randomNumber);
        }

        /**
         * Change the src of a random image element with another image.
         */
        function changeRandomImage() {
          // Generate random number within limit of `imageListUrls.length`
          const randomImageNumber = Math.floor(Math.random() * imageListUrls.length);

          // Get always UNIQUE random number from `imgElements` array.
          const randomImgNumber = Math.floor(Math.random() * imgElements.length);
          const randomImgElement = imgElements[randomImgNumber];

          // If `visibleImages` array contains `randomImageNumber` calculate
          // random image again, otherwise image `scr` URL.
          if (visibleImages.includes(randomImageNumber)) {
            changeRandomImage();
          } else {
            visibleImages[randomImgNumber] = randomImageNumber;
            randomImgElement.src = imageListUrls[randomImageNumber];
            randomImgElement.animate([{ opacity: 0 }, { opacity: 1 }], {
              duration: 2000,
              fill: "forwards"
            });
          }
        }

        // Change url on random image per defined change interval.
        if (availableImagesCount > displayableImagesCount) {
          setInterval(changeRandomImage, changeInterval);
        }
      });
    }
  };
})(Drupal);
